## DPDK for Mellanox Bluefield-1 ARM cores 

### Install and configure DPDK

1. Download the required DPDK compressed version & decompress it. 

    https://core.dpdk.org/download/
    
    It is good to choose a stable (LTS) version 
2. For Mellanox Bluefield NICs, check if the flag for Mellanox driver installation is set (for DPDK version < 20.11.8 only)

    <path-to-dpdk-folder>$ vim config/common_linux

    a. Check if the Poll Mode Driver (PMD) for your Mellanox NIC is set; in our case it is MLX5 
    b. Add “CONFIG_RTE_LIBRTE_MLX5_PMD=y”, if it does not exist

3. DPDK configuration & alternatives
    - Alternative 1: For newer DPDK versions (>= version 18)
    
        <path-to-dpdk-folder>$ meson build

        <path-to-dpdk-folder>$ cd build
        
        <path-to-dpdk-folder>$ ninja
        
        <path-to-dpdk-folder>$ ninja install
    - Alternative 2: For older DPDK versions & for our mTCP setup
        - Run: 
            
            <path-to-dpdk-folder>/usertools$ sudo ./dpdk-setup.sh
        - Choose 
            
            7: arm64-bluefield-linuxapp-gcc (NIC installation)
            
            41: x86_64-native-linuxapp-gcc (Host installation)
            
            Corresponding folder is created at DPDK root folder
    - For both alternatives:
        - Mellanox driver, mlx5 supports kernel and poll mode
        - No need to unbind the interface from kernel
        - Allocate huge pages for DPDK operation
            
            $ sudo sysctl -w vm.nr_hugepages=4096
        - Test huge page allocation
            $ cat /proc/meminfo | grep -i huge

**Note: In case of numa.h not found error**
    
$ sudo apt-get install libnuma-dev

### Run DPDK application

1. Identify PCI address of the NIC port which you want to bind to DPDK 
    
    $ lspci

    03:00.0 Ethernet controller: Mellanox Technologies MT416842 BlueField integrated ConnectX-5 network controller => p0 
    
    03:00.1 Ethernet controller: Mellanox Technologies MT416842 BlueField integrated ConnectX-5 network controller => p1
2. Run testpmd on the two smartNICs connected directly using a cable

    <path-to-dpdk-folder>/arm64-bluefield-linuxapp-gcc/build/app/test-pmd$ sudo ./testpmd -w 03:00.1 -- --nb-cores=2 --txq=2 -i 

    **Parameters explained:** 
    
    w: whitelist device; nb-cores: #forwarding cores; txq: #transmit queues (requires RSS); i: interactive mode; a: automatic traffic start 

    You can check for more configuration parameters: testpmd> help 

3. In the interactive mode, some configurations before testing network bandwidth

    **I have considered our servers "ub-05" and "user" to demonstrate example configuration**
    - “ub-05” server smartNIC 
        
        testpmd> set fwd txonly 
        
        testpmd> set eth-peer 0 0c:42:a1:df:ac:41
        
        testpmd> set eth-peer 1 0c:42:a1:df:ac:40

        testpmd> start
        
        testpmd> show port stats all //repeat 
    - “user” server smartNIC 
        
        testpmd> set fwd rxonly 

        testpmd> set eth-peer 0 0c:42:a1:df:ac:49

        testpmd> set eth-peer 1 0c:42:a1:df:ac:48

        testpmd> start

        testpmd> show port stats all //repeat 
        



